@@ -17,10 +17,18 @@ from robot.model.visitor import SuiteVisitor
 from robot.output import LOGGER, Message
 from robot.running.builder import TestSuiteBuilder
 from robot.running.builder.builders import SuiteStructureParser
+from robot.utils import NormalizedDict, normalize
 from robot.utils.filereader import FileReader
 from robotcode.core.dataclasses import from_json
-from robotcode.core.lsp.types import Diagnostic, DiagnosticSeverity, DocumentUri, Position, Range
+from robotcode.core.lsp.types import (
+    Diagnostic,
+    DiagnosticSeverity,
+    DocumentUri,
+    Position,
+    Range,
+)
 from robotcode.core.uri import Uri
+from robotcode.core.utils.cli import show_hidden_arguments
 from robotcode.plugin import Application, OutputFormat, UnknownError, pass_application
 from robotcode.plugin.click_helper.types import add_options
 from robotcode.robot.utils import get_robot_version
@@ -47,9 +55,13 @@ def _patch() -> None:
 
     if get_robot_version() <= (6, 1):
         if get_robot_version() > (5, 0) and get_robot_version() < (6, 0) or get_robot_version() < (5, 0):
-            from robot.running.builder.testsettings import TestDefaults  # pyright: ignore[reportMissingImports]
+            from robot.running.builder.testsettings import (  # pyright: ignore[reportMissingImports]
+                TestDefaults,
+            )
         else:
-            from robot.running.builder.settings import Defaults as TestDefaults  # pyright: ignore[reportMissingImports]
+            from robot.running.builder.settings import (  #  pyright: ignore[reportMissingImports]
+                Defaults as TestDefaults,
+            )
 
         old_validate_test_counts = TestSuiteBuilder._validate_test_counts
 
@@ -74,11 +86,15 @@ def _patch() -> None:
                     from robot.running.builder.parsers import format_name
 
                     return ErroneousTestSuite(
-                        error_message=str(e), name=format_name(structure.source), source=structure.source
+                        error_message=str(e),
+                        name=format_name(structure.source),
+                        source=structure.source,
                     ), TestDefaults(parent_defaults)
 
                 return ErroneousTestSuite(
-                    error_message=str(e), name=TestSuite.name_from_source(structure.source), source=structure.source
+                    error_message=str(e),
+                    name=TestSuite.name_from_source(structure.source),
+                    source=structure.source,
                 ), TestDefaults(parent_defaults)
 
         SuiteStructureParser._build_suite = build_suite
@@ -95,7 +111,9 @@ def _patch() -> None:
 
     elif get_robot_version() >= (6, 1):
         from robot.parsing.suitestructure import SuiteDirectory, SuiteFile
-        from robot.running.builder.settings import TestDefaults  # pyright: ignore[reportMissingImports]
+        from robot.running.builder.settings import (  # pyright: ignore[reportMissingImports]
+            TestDefaults,
+        )
 
         old_validate_not_empty = TestSuiteBuilder._validate_not_empty
 
@@ -115,7 +133,9 @@ def _patch() -> None:
             except DataError as e:
                 LOGGER.error(str(e))
                 return ErroneousTestSuite(
-                    error_message=str(e), name=TestSuite.name_from_source(structure.source), source=structure.source
+                    error_message=str(e),
+                    name=TestSuite.name_from_source(structure.source),
+                    source=structure.source,
                 )
 
         SuiteStructureParser._build_suite_file = build_suite_file
@@ -130,7 +150,9 @@ def _patch() -> None:
             except DataError as e:
                 LOGGER.error(str(e))
                 return ErroneousTestSuite(
-                    error_message=str(e), name=TestSuite.name_from_source(structure.source), source=structure.source
+                    error_message=str(e),
+                    name=TestSuite.name_from_source(structure.source),
+                    source=structure.source,
                 ), TestDefaults(self.parent_defaults)
 
         SuiteStructureParser._build_suite_directory = build_suite_directory
@@ -213,9 +235,24 @@ class Collector(SuiteVisitor):
         self.suites: List[TestItem] = []
         self.tests: List[TestItem] = []
         self.tags: Dict[str, List[TestItem]] = defaultdict(list)
+        self.normalized_tags: Dict[str, List[TestItem]] = defaultdict(list)
         self.statistics = Statistics()
+        self._collected = [NormalizedDict(ignore="_")]
 
     def visit_suite(self, suite: TestSuite) -> None:
+        if suite.name in self._collected[-1] and suite.parent.source:
+            LOGGER.warn(
+                (
+                    f"Warning in {'file' if Path(suite.parent.source).is_file() else 'folder'} "
+                    f"'{suite.parent.source}': "
+                    if suite.source and Path(suite.parent.source).exists()
+                    else ""
+                )
+                + f"Multiple suites with name '{suite.name}' in suite '{suite.parent.longname}'."
+            )
+
+        self._collected[-1][suite.name] = True
+        self._collected.append(NormalizedDict(ignore="_"))
         try:
             item = TestItem(
                 type="suite",
@@ -253,7 +290,18 @@ class Collector(SuiteVisitor):
         if suite.tests:
             self.statistics.suites_with_tests += 1
 
+    def end_suite(self, _suite: TestSuite) -> None:
+        self._collected.pop()
+
     def visit_test(self, test: TestCase) -> None:
+        if test.name in self._collected[-1]:
+            LOGGER.warn(
+                f"Warning in file '{test.source}' on line {test.lineno}: "
+                f"Multiple {'task' if test.parent.rpa else 'test'}s with name '{test.name}' in suite "
+                f"'{test.parent.longname}'."
+            )
+        self._collected[-1][test.name] = True
+
         if self._current.children is None:
             self._current.children = []
         try:
@@ -275,6 +323,7 @@ class Collector(SuiteVisitor):
 
         for tag in test.tags:
             self.tags[str(tag)].append(item)
+            self.normalized_tags[normalize(str(tag), ignore="_")].append(item)
 
         self.tests.append(item)
         self._current.children.append(item)
@@ -284,7 +333,10 @@ class Collector(SuiteVisitor):
 
 @click.group(invoke_without_command=False)
 @click.option(
-    "--read-from-stdin", is_flag=True, help="Read file contents from stdin. This is an internal option.", hidden=True
+    "--read-from-stdin",
+    is_flag=True,
+    help="Read file contents from stdin. This is an internal option.",
+    hidden=show_hidden_arguments(),
 )
 @pass_application
 def discover(app: Application, read_from_stdin: bool) -> None:
@@ -304,7 +356,9 @@ def discover(app: Application, read_from_stdin: bool) -> None:
         app.verbose(f"Read data from stdin: {_stdin_data!r}")
 
 
-RE_IN_FILE_LINE_MATCHER = re.compile(r".+\sin\sfile\s'(?P<file>.*)'\son\sline\s(?P<line>\d+):(?P<message>.*)")
+RE_IN_FILE_LINE_MATCHER = re.compile(
+    r".+\sin\s(file|folder)\s'(?P<file>.*)'(\son\sline\s(?P<line>\d+))?:(?P<message>.*)"
+)
 RE_PARSING_FAILED_MATCHER = re.compile(r"Parsing\s'(?P<file>.*)'\sfailed:(?P<message>.*)")
 
 
@@ -321,7 +375,10 @@ def build_diagnostics(messages: List[Message]) -> Dict[str, List[Diagnostic]]:
     result: Dict[str, List[Diagnostic]] = {}
 
     def add_diagnostic(
-        message: Message, source_uri: Optional[str] = None, line: Optional[int] = None, text: Optional[str] = None
+        message: Message,
+        source_uri: Optional[str] = None,
+        line: Optional[int] = None,
+        text: Optional[str] = None,
     ) -> None:
         source_uri = str(Uri.from_path(Path(source_uri).resolve() if source_uri else Path.cwd()))
 
@@ -343,7 +400,12 @@ def build_diagnostics(messages: List[Message]) -> Dict[str, List[Diagnostic]]:
 
     for message in messages:
         if match := RE_IN_FILE_LINE_MATCHER.match(message.message):
-            add_diagnostic(message, match.group("file"), int(match.group("line")), text=match.group("message").strip())
+            add_diagnostic(
+                message,
+                match.group("file"),
+                int(match.group("line")) if match.group("line") is not None else None,
+                text=match.group("message").strip(),
+            )
         elif match := RE_PARSING_FAILED_MATCHER.match(message.message):
             add_diagnostic(message, match.group("file"), text=match.group("message").strip())
         else:
@@ -354,10 +416,11 @@ def build_diagnostics(messages: List[Message]) -> Dict[str, List[Diagnostic]]:
 
 def handle_options(
     app: Application,
+    diagnostics: bool,
     by_longname: Tuple[str, ...],
     exclude_by_longname: Tuple[str, ...],
     robot_options_and_args: Tuple[str, ...],
-) -> Tuple[TestSuite, Optional[Dict[str, List[Diagnostic]]]]:
+) -> Tuple[TestSuite, Collector, Optional[Dict[str, List[Diagnostic]]]]:
     root_folder, profile, cmd_options = handle_robot_options(
         app, by_longname, exclude_by_longname, robot_options_and_args
     )
@@ -378,7 +441,10 @@ def handle_options(
 
         settings = RobotSettings(options)
 
-        LOGGER.register_console_logger(**settings.console_output_config)
+        if diagnostics:
+            LOGGER.register_console_logger(**settings.console_output_config)
+        else:
+            LOGGER.unregister_console_logger()
 
         diagnostics_logger = DiagnosticsLogger()
         LOGGER.register_logger(diagnostics_logger)
@@ -418,7 +484,10 @@ def handle_options(
             suite.visit(ModelModifier(settings.pre_run_modifiers, settings.run_empty_suite, LOGGER))
         suite.configure(**settings.suite_config)
 
-        return suite, build_diagnostics(diagnostics_logger.messages)
+        collector = Collector()
+        suite.visit(collector)
+
+        return suite, collector, build_diagnostics(diagnostics_logger.messages)
 
     except Information as err:
         app.echo(str(err))
@@ -430,6 +499,16 @@ def handle_options(
     raise UnknownError("Unexpected error happened.")
 
 
+DIAGOSTICS_OPTIONS = {
+    click.option(
+        "--diagnostics / --no-diagnostics",
+        "show_diagnostics",
+        default=True,
+        help="Display `robot` parsing errors and warning that occur during discovering.",
+    )
+}
+
+
 @discover.command(
     context_settings={
         "allow_extra_args": True,
@@ -438,10 +517,12 @@ def handle_options(
     add_help_option=True,
     epilog='Use "-- --help" to see `robot` help.',
 )
+@add_options(*DIAGOSTICS_OPTIONS)
 @add_options(*ROBOT_OPTIONS)
 @pass_application
 def all(
     app: Application,
+    show_diagnostics: bool,
     by_longname: Tuple[str, ...],
     exclude_by_longname: Tuple[str, ...],
     robot_options_and_args: Tuple[str, ...],
@@ -450,18 +531,20 @@ def all(
     Discover suites, tests, tasks with the selected configuration,
     profiles, options and arguments.
 
+    You can use all known `robot` arguments to filter for example by tags or to use pre-run-modifier.
+
     \b
     Examples:
     ```
     robotcode discover all
     robotcode --profile regression discover all
+    robotcode --profile regression discover all --include regression --exclude wipANDnotready
     ```
     """
 
-    suite, diagnostics = handle_options(app, by_longname, exclude_by_longname, robot_options_and_args)
-
-    collector = Collector()
-    suite.visit(collector)
+    suite, collector, diagnostics = handle_options(
+        app, show_diagnostics, by_longname, exclude_by_longname, robot_options_and_args
+    )
 
     if collector.all.children:
         if app.config.output_format is None or app.config.output_format == OutputFormat.TEXT:
@@ -469,7 +552,8 @@ def all(
 
             def print(item: TestItem, indent: int = 0) -> Iterable[str]:
                 type = click.style(
-                    item.type.capitalize() if item.type == "suite" else tests_or_tasks.capitalize(), fg="green"
+                    item.type.capitalize() if item.type == "suite" else tests_or_tasks.capitalize(),
+                    fg="green",
                 )
 
                 if item.type == "test":
@@ -509,10 +593,12 @@ def all(
     add_help_option=True,
     epilog='Use "-- --help" to see `robot` help.',
 )
+@add_options(*DIAGOSTICS_OPTIONS)
 @add_options(*ROBOT_OPTIONS)
 @pass_application
 def tests(
     app: Application,
+    show_diagnostics: bool,
     by_longname: Tuple[str, ...],
     exclude_by_longname: Tuple[str, ...],
     robot_options_and_args: Tuple[str, ...],
@@ -521,18 +607,20 @@ def tests(
     Discover tests with the selected configuration, profiles, options and
     arguments.
 
+    You can use all known `robot` arguments to filter for example by tags or to use pre-run-modifier.
+
     \b
     Examples:
     ```
     robotcode discover tests
     robotcode --profile regression discover tests
+    robotcode --profile regression discover tests --include regression --exclude wipANDnotready
     ```
     """
 
-    suite, diagnostics = handle_options(app, by_longname, exclude_by_longname, robot_options_and_args)
-
-    collector = Collector()
-    suite.visit(collector)
+    suite, collector, diagnostics = handle_options(
+        app, show_diagnostics, by_longname, exclude_by_longname, robot_options_and_args
+    )
 
     if collector.all.children:
         if app.config.output_format is None or app.config.output_format == OutputFormat.TEXT:
@@ -556,10 +644,12 @@ def tests(
     add_help_option=True,
     epilog='Use "-- --help" to see `robot` help.',
 )
+@add_options(*DIAGOSTICS_OPTIONS)
 @add_options(*ROBOT_OPTIONS)
 @pass_application
 def suites(
     app: Application,
+    show_diagnostics: bool,
     by_longname: Tuple[str, ...],
     exclude_by_longname: Tuple[str, ...],
     robot_options_and_args: Tuple[str, ...],
@@ -568,18 +658,20 @@ def suites(
     Discover suites with the selected configuration, profiles, options and
     arguments.
 
+    You can use all known `robot` arguments to filter for example by tags or to use pre-run-modifier.
+
     \b
     Examples:
     ```
     robotcode discover suites
     robotcode --profile regression discover suites
+    robotcode --profile regression discover suites --include regression --exclude wipANDnotready
     ```
     """
 
-    suite, diagnostics = handle_options(app, by_longname, exclude_by_longname, robot_options_and_args)
-
-    collector = Collector()
-    suite.visit(collector)
+    suite, collector, diagnostics = handle_options(
+        app, show_diagnostics, by_longname, exclude_by_longname, robot_options_and_args
+    )
 
     if collector.all.children:
         if app.config.output_format is None or app.config.output_format == OutputFormat.TEXT:
@@ -608,10 +700,19 @@ class TagsResult:
     add_help_option=True,
     epilog='Use "-- --help" to see `robot` help.',
 )
+@click.option(
+    "--normalized / --not-normalized",
+    "normalized",
+    default=True,
+    help="Whether or not normalized tags are shown.",
+)
+@add_options(*DIAGOSTICS_OPTIONS)
 @add_options(*ROBOT_OPTIONS)
 @pass_application
 def tags(
     app: Application,
+    show_diagnostics: bool,
+    normalized: bool,
     by_longname: Tuple[str, ...],
     exclude_by_longname: Tuple[str, ...],
     robot_options_and_args: Tuple[str, ...],
@@ -620,6 +721,8 @@ def tags(
     Discover tags with the selected configuration, profiles, options and
     arguments.
 
+    You can use all known `robot` arguments to filter for example by tags or to use pre-run-modifier.
+
     \b
     Examples:
     ```
@@ -630,10 +733,9 @@ def tags(
     ```
     """
 
-    suite, _diagnostics = handle_options(app, by_longname, exclude_by_longname, robot_options_and_args)
-
-    collector = Collector()
-    suite.visit(collector)
+    _suite, collector, _diagnostics = handle_options(
+        app, show_diagnostics, by_longname, exclude_by_longname, robot_options_and_args
+    )
 
     if collector.all.children:
         if app.config.output_format is None or app.config.output_format == OutputFormat.TEXT:
@@ -641,12 +743,14 @@ def tags(
             def print(tags: Dict[str, List[TestItem]]) -> Iterable[str]:
                 for tag, items in tags.items():
                     yield f"{tag}{os.linesep}"
+                    # for t in items:
+                    #     yield f"    {t.longname}{os.linesep}"
 
-            if collector.suites:
-                app.echo_via_pager(print(collector.tags))
+            if collector.normalized_tags:
+                app.echo_via_pager(print(collector.normalized_tags if normalized else collector.tags))
 
         else:
-            app.print_data(TagsResult(collector.tags), remove_defaults=True)
+            app.print_data(TagsResult(collector.normalized_tags), remove_defaults=True)
 
 
 @dataclass
